package dev.pfaff.altargui.util.listenable;

import it.unimi.dsi.fastutil.objects.ObjectArraySet;

import java.util.Set;
import java.util.function.Consumer;

public abstract class AbstractListenable implements Listenable {
	private final Set<Object> listeners = new ObjectArraySet<>();

	protected final boolean addListener(Object listener) {
		return listeners.add(listener);
	}

	protected final void removeListener(Object listener) {
		listeners.remove(listener);
	}

	protected final void forEachListener(Consumer<Object> consumer) {
		listeners.forEach(consumer);
	}

	protected final void notifyRunnableListeners() {
		forEachListener(listener -> ((Runnable) listener).run());
	}

	@Override
	public final ListenerDisposer addListener(Runnable listener) {
		if (addListener((Object)listener)) {
			listener.run();
		}
		return () -> removeListener(listener);
	}
}
