package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.StateContext;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.WidgetInstance;
import dev.pfaff.altargui.WidgetInstantiationArgs;
import dev.pfaff.altargui.element.StatefulWidgetInstance;
import dev.pfaff.altargui.util.listenable.Listenable;
import dev.pfaff.altargui.util.listenable.ListenerDisposer;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.function.Supplier;

/**
 * A widget that replaces its child using the provided {@code builder} whenever notified by listenable.
 */
public record ListenableBuilderWidget(Listenable listenable, Supplier<Widget> builder) implements Widget {
	@Override
	public @NotNull WidgetInstance<ListenableBuilderWidget> instantiate(WidgetInstantiationArgs args) {
		return new State(args);
	}

	private static final class State extends StatefulWidgetInstance<ListenableBuilderWidget> {
		private Listenable listenable;
		private Supplier<Widget> builder;

		private @Nullable ListenerDisposer listener;

		private State(WidgetInstantiationArgs args) {
			super(args);
		}

		@Override
		protected @NotNull Widget build() {
			return builder.get();
		}

		@Override
		public void configure(StateContext stateContext, ListenableBuilderWidget widget, boolean init) {
			if (builder != widget.builder) {
				builder = widget.builder;
				markNeedsRebuild();
			}
			if (listenable != widget.listenable) {
				if (listener != null) listener.close();
				var listenable = this.listenable = widget.listenable;
				listener = listenable.addListener(this::markNeedsRebuild);
			}
		}

		@Override
		public void dispose() {
			if (listener != null) {
				listener.close();
				listener = null;
			}
			super.dispose();
		}
	}
}
