package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.function.CharProvider;
import dev.pfaff.altargui.text.Font;
import dev.pfaff.altargui.types.Color;
import dev.pfaff.altargui.util.listenable.ConstantValue;
import dev.pfaff.altargui.util.listenable.ValueListenable;
import org.jetbrains.annotations.NotNull;

import java.lang.invoke.MethodHandle;
import java.lang.invoke.MethodHandles;
import java.lang.invoke.MethodType;
import java.util.function.IntUnaryOperator;

import static dev.pfaff.altargui.GuiGlobals.defaultFont;
import static dev.pfaff.altargui.GuiGlobals.WINDOW;
import static dev.pfaff.altargui.text.StyleFlags.*;
import static dev.pfaff.altargui.util.Bits.bit;
import static dev.pfaff.altargui.util.MethodHandleUtil.findStatic;

public sealed interface TextWidget extends Widget permits TextFragmentWidget {
	static final float DEFAULT_SCALED_FONT_SIZE = 8f;
	static final ValueListenable<Float> DEFAULT_SCALED_FONT_SIZE_LISTENABLE =
//			windowScaleFactorProxy.map(f -> f * DEFAULT_SCALED_FONT_SIZE);
			new ConstantValue<>(DEFAULT_SCALED_FONT_SIZE);

	static final TextFragmentWidget EMPTY = new TextFragmentWidget(CharProvider.ofNone(), defaultFont());

	static final MethodHandle MH_ofString = findStatic(MethodHandles.lookup(),
													   TextWidget.class,
													   "of",
													   MethodType.methodType(TextWidget.class, String.class));
	TextWidget[] EMPTY_ARRAY = new TextWidget[0];

	static TextWidget of(@NotNull String text) {
		if (text.isEmpty()) return EMPTY;
		return new TextFragmentWidget(text, defaultFont());
	}

	TextWidget withFont(Font font);

	ValueListenable<Float> fontSize();

	TextWidget withFontSize(ValueListenable<Float> fontSize);

	default TextWidget withFontSize(float size) {
		return withFontSize(WINDOW().scaleFactorListenable().map(scaleFactor -> size / scaleFactor));
	}

	default TextWidget withScaledFontSize(float size) {
		return withFontSize(new ConstantValue<>(size));
	}

	TextWidget withColor(Color color);

	TextWidget mapFlags(IntUnaryOperator modifier);

	default TextWidget withFlags(int flags) {
		return mapFlags(_ -> flags);
	}

	default TextWidget withBold(boolean bold) {
		return mapFlags(flags -> bit(flags, FLAG_BOLD, bold));
	}
	default TextWidget withItalic(boolean italic) {
		return mapFlags(flags -> bit(flags, FLAG_ITALIC, italic));
	}
	default TextWidget withStrike(boolean strike) {
		return mapFlags(flags -> bit(flags, FLAG_STRIKE, strike));
	}
	default TextWidget withUnderline(boolean underline) {
		return mapFlags(flags -> bit(flags, FLAG_UNDERLINE, underline));
	}
}
